﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static LinkedList;

namespace KursiveWork;

// Класс-хранилище состояний списка
public class LinkedListStorage
{
    public List<LinkedListState> states;
    public int currentIndex;

    public LinkedListStorage()
    {
        states = new List<LinkedListState>();
        currentIndex = -1;  // Инициализация currentIndex
        
    }

    
    // Метод добавления нового состояния
    public void AddState(LinkedListState state)
    {
        states.Add(state);
        currentIndex++;
    }

    
    public LinkedListState GetNextState()
    {
        if (currentIndex < states.Count - 1)
        {
            currentIndex++;
            return states[currentIndex];
        }
        else
        {
            return null;
        }
    }

    // Метод получения предыдущего состояния
    public LinkedListState GetPreviousState()
    {
        if (currentIndex > 0) // Проверка на выход за границы
        {
            currentIndex--;
            return states[currentIndex];
        }
        else
        {
            return null;
        }
    }

    // Метод возврата в начальное состояние
    public LinkedListState Reset()
    {
        currentIndex = 0;

        // Проверка, не пуст ли список состояний
        if (states.Count > 0)
        {
            return states[currentIndex]; // Возвращаем первое состояние
        }
        else
        {
            return null; // Возвращаем null, если список пуст
        }
    }

    // Метод сохранения состояний в файл
    

    // Вспомогательный метод для сохранения узла
    private void SaveNode(BinaryWriter writer, Node node)
    {
        if (node != null)
        {
            writer.Write(node.Data); // Записываем данные узла в двоичном формате
            SaveNode(writer, node.Next);
        }
        else
        {
            writer.Write(-1); // Используем -1 как маркер конца списка
        }
    }

    public void Save(string filename)
    {
        using (BinaryWriter writer = new BinaryWriter(File.Open(filename, FileMode.Create)))
        {
            writer.Write(states.Count); // Записываем количество состояний в двоичном формате
            foreach (LinkedListState state in states)
            {
                SaveNode(writer, state.Head);
            }
        }
    }
    public List<LinkedListState> Load(string filename)
    {
        //states.Clear();
        currentIndex = 0;

        using (BinaryReader reader = new BinaryReader(File.Open(filename, FileMode.Open)))
        {
            int count = reader.ReadInt32(); // Читаем количество состояний
            for (int i = 0; i < count; i++)
            {
                states.Add(new LinkedListState(LoadNode(reader)));
            }
        }
        currentIndex = states.Count - 1;
        return states;
    }

    private Node LoadNode(BinaryReader reader)
    {
        int data = reader.ReadInt32(); // Читаем данные узла
        if (data == -1)
        {
            return null;
        }
        else
        {
            return new Node(data) { Next = LoadNode(reader) };
        }
    }

    
}


