﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static LinkedList;
using System.Drawing;


namespace KursiveWork
{
    public partial class MainForm : Form
    {
        private LinkedList list;
        private LinkedListVisualizer visualizer;
        private LinkedListManager manager;
        private LinkedListParameters parameters;


        public MainForm()
        {
            InitializeComponent();
            list = new LinkedList();
            visualizer = new LinkedListVisualizer();
            
            
            //MessageBox.Show("Настоятельно рекомендуется ознакомиться с информацией с помощью меню сверху");
            manager = new LinkedListManager(parameters);
            Controls.Add(pictureBox);
            pictureBox.Image = visualizer.Visualize(list.CreateState());
            manager.storage.AddState(list.CreateState());
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {

            Inputform inputform = new Inputform();
            inputform.ShowDialog();


            // Получение значения из текстового поля
            if (inputform.Number != null)
            {
                // Преобразование в целое число успешно
                // Добавляем элемент в список
                int Number = (int)inputform.Number;
                list.InsertAtBeginning(Number);

                manager.storage.AddState(list.CreateState());
                
                pictureBox.Image = visualizer.Visualize(list.CreateState());

            }
            else
            {
                // Ошибка преобразования, показать сообщение об ошибке
                MessageBox.Show("Некорректный ввод данных!");
            }

        }

        private void ButtonSearch_Click(object sender, EventArgs e)
        {
            Inputform inputform = new Inputform();
            inputform.ShowDialog();

            // Получение значения из текстового поля
            if (inputform.Number != null)
            {
                int Number = (int)inputform.Number;
                // Преобразование в целое число успешно
                // Выполняем поиск элемента в списке
                bool found = list.Search(Number);

                // Отображаем результат поиска
                if (found)
                {
                    MessageBox.Show("Элемент найден!");
                }
                else
                {
                    MessageBox.Show("Элемент не найден!");
                }
            }
            else
            {
                // Ошибка преобразования, показать сообщение об ошибке
                MessageBox.Show("Некорректный ввод данных!");
            }

        }

        private void buttonNextStep_Click(object sender, EventArgs e)
        {
            LinkedListState state = manager.storage.GetNextState();
            if (state != null)
            {
                list.RestoreState(state); // Восстановление состояния списка
                pictureBox.Image = visualizer.Visualize(state); // Обновление визуализации
            }
        }

        private void buttonPreviousStep_Click(object sender, EventArgs e)
        {
            LinkedListState state = manager.storage.GetPreviousState();
            if (state != null)
            {
                list.RestoreState(state); // Восстановление состояния списка
                pictureBox.Image = visualizer.Visualize(state); // Обновление визуализации
            }
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.Filter = "Файл состояний (*.dat)|*.dat"; // Изменяем фильтр на *.txt
            saveFileDialog.Title = "Сохранить состояния";
            saveFileDialog.ShowDialog();

            if (saveFileDialog.FileName != "")
            {
                manager.storage.Save(saveFileDialog.FileName); // Сохраняем в текстовый файл
                MessageBox.Show("Файл сохранен");
            }
            else MessageBox.Show("Ошибка сохранения");
        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "Файл состояний (*.dat)|*.dat"; // Изменяем фильтр на *.txt
            openFileDialog.Title = "Загрузить состояния";
            openFileDialog.ShowDialog();

            if (openFileDialog.FileName != "")
            {
                List<LinkedListState> loadedStates = manager.storage.Load(openFileDialog.FileName); // Загружаем из текстового файла

                // Получаем первое состояние из списка
                LinkedListState firstState = loadedStates[0];

                // Восстанавливаем первое состояние в списке list
                list.RestoreState(firstState);
                manager.storage.currentIndex = 0;
                for (int i = 0; i < loadedStates.Count; i++)
                {
                    
                }
                pictureBox.Image = visualizer.Visualize(firstState);
            }
        }

        private void InfoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            infoform infoform = new infoform();
            infoform.ShowDialog();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            manager.storage.currentIndex = 1;
            list.RestoreState(manager.storage.GetPreviousState());
            manager.storage.currentIndex = 1;
            LinkedListState state = manager.storage.GetPreviousState();
            pictureBox.Image = visualizer.Visualize(state);
        }
    }
}